@@ -7,44 +7,54 @@
 
     Gracefully adapted from the TextPress system by Armin.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import print_function
 
 import os
 import sys
 import types
 import posixpath
+import traceback
 from os import path
-from io import StringIO
+from collections import deque
 
+from six import iteritems, itervalues, text_type
+from six.moves import cStringIO
 from docutils import nodes
 from docutils.parsers.rst import convert_directive_function, \
-     directives, roles
+    directives, roles
 
 import sphinx
 from sphinx import package_dir, locale
 from sphinx.roles import XRefRole
 from sphinx.config import Config
 from sphinx.errors import SphinxError, SphinxWarning, ExtensionError, \
-     VersionRequirementError
+    VersionRequirementError, ConfigError
 from sphinx.domains import ObjType, BUILTIN_DOMAINS
 from sphinx.domains.std import GenericObject, Target, StandardDomain
 from sphinx.builders import BUILTIN_BUILDERS
 from sphinx.environment import BuildEnvironment, SphinxStandaloneReader
-from sphinx.util import pycompat  # imported for side-effects
+from sphinx.util import pycompat  # noqa: imported for side-effects
+from sphinx.util import import_object
 from sphinx.util.tags import Tags
 from sphinx.util.osutil import ENOENT
-from sphinx.util.console import bold, lightgray, darkgray
+from sphinx.util.console import bold, lightgray, darkgray, darkgreen, \
+    term_width_line
 
+if hasattr(sys, 'intern'):
+    intern = sys.intern
 
 # List of all known core events. Maps name to arguments description.
 events = {
     'builder-inited': '',
     'env-get-outdated': 'env, added, changed, removed',
     'env-purge-doc': 'env, docname',
+    'env-before-read-docs': 'env, docnames',
     'source-read': 'docname, source text',
     'doctree-read': 'the doctree before being pickled',
+    'env-merge-info': 'env, read docnames, other env instance',
     'missing-reference': 'env, node, contnode',
     'doctree-resolved': 'doctree, docname',
     'env-updated': 'env',
@@ -66,6 +76,7 @@ class Sphinx(object):
         self.verbosity = verbosity
         self.next_listener_id = 0
         self._extensions = {}
+        self._extension_metadata = {}
         self._listeners = {}
         self.domains = BUILTIN_DOMAINS.copy()
         self.builderclasses = BUILTIN_BUILDERS.copy()
@@ -80,59 +91,97 @@ class Sphinx(object):
         self.parallel = parallel
 
         if status is None:
-            self._status = StringIO()
+            self._status = cStringIO()
             self.quiet = True
         else:
             self._status = status
             self.quiet = False
 
         if warning is None:
-            self._warning = StringIO()
+            self._warning = cStringIO()
         else:
             self._warning = warning
         self._warncount = 0
         self.warningiserror = warningiserror
 
         self._events = events.copy()
+        self._translators = {}
+
+        # keep last few messages for traceback
+        self.messagelog = deque(maxlen=10)
 
         # say hello to the world
-        self.info(bold('Running Sphinx v%s' % sphinx.__version__))
+        self.info(bold('Running Sphinx v%s' % sphinx.__display_version__))
 
         # status code for command-line application
         self.statuscode = 0
 
+        if not path.isdir(outdir):
+            self.info('making output directory...')
+            os.makedirs(outdir)
+
         # read config
         self.tags = Tags(tags)
         self.config = Config(confdir, CONFIG_FILENAME,
                              confoverrides or {}, self.tags)
         self.config.check_unicode(self.warn)
+        # defer checking types until i18n has been initialized
 
         # set confdir to srcdir if -C given (!= no confdir); a few pieces
         # of code expect a confdir to be set
         if self.confdir is None:
             self.confdir = self.srcdir
 
-        # backwards compatibility: activate old C markup
-        self.setup_extension('sphinx.ext.oldcmarkup')
+        # extension loading support for alabaster theme
+        # self.config.html_theme is not set from conf.py at here
+        # for now, sphinx always load a 'alabaster' extension.
+        if 'alabaster' not in self.config.extensions:
+            self.config.extensions.append('alabaster')
+
         # load all user-given extension modules
         for extension in self.config.extensions:
             self.setup_extension(extension)
         # the config file itself can be an extension
         if self.config.setup:
-            self.config.setup(self)
+            # py31 doesn't have 'callable' function for below check
+            if hasattr(self.config.setup, '__call__'):
+                self.config.setup(self)
+            else:
+                raise ConfigError(
+                    "'setup' that is specified in the conf.py has not been " +
+                    "callable. Please provide a callable `setup` function " +
+                    "in order to behave as a sphinx extension conf.py itself."
+                )
 
         # now that we know all config values, collect them from conf.py
-        self.config.init_values()
+        self.config.init_values(self.warn)
 
         # check the Sphinx version if requested
         if self.config.needs_sphinx and \
-           self.config.needs_sphinx > sphinx.__version__[:3]:
+           self.config.needs_sphinx > sphinx.__display_version__[:3]:
             raise VersionRequirementError(
                 'This project needs at least Sphinx v%s and therefore cannot '
                 'be built with this version.' % self.config.needs_sphinx)
 
+        # check extension versions if requested
+        if self.config.needs_extensions:
+            for extname, needs_ver in self.config.needs_extensions.items():
+                if extname not in self._extensions:
+                    self.warn('needs_extensions config value specifies a '
+                              'version requirement for extension %s, but it is '
+                              'not loaded' % extname)
+                    continue
+                has_ver = self._extension_metadata[extname]['version']
+                if has_ver == 'unknown version' or needs_ver > has_ver:
+                    raise VersionRequirementError(
+                        'This project needs the extension %s at least in '
+                        'version %s and therefore cannot be built with the '
+                        'loaded version (%s).' % (extname, needs_ver, has_ver))
+
         # set up translation infrastructure
         self._init_i18n()
+        # check all configuration values for permissible types
+        self.config.check_types(self.warn)
         # set up the build environment
         self._init_env(freshenv)
         # set up the builder
@@ -163,20 +212,20 @@ class Sphinx(object):
             self.env = BuildEnvironment(self.srcdir, self.doctreedir,
                                         self.config)
             self.env.find_files(self.config)
-            for domain in list(self.domains.keys()):
+            for domain in self.domains.keys():
                 self.env.domains[domain] = self.domains[domain](self.env)
         else:
             try:
                 self.info(bold('loading pickled environment... '), nonl=True)
-                self.env = BuildEnvironment.frompickle(self.config,
-                    path.join(self.doctreedir, ENV_PICKLE_FILENAME))
+                self.env = BuildEnvironment.frompickle(
+                    self.config, path.join(self.doctreedir, ENV_PICKLE_FILENAME))
                 self.env.domains = {}
-                for domain in list(self.domains.keys()):
+                for domain in self.domains.keys():
                     # this can raise if the data version doesn't fit
                     self.env.domains[domain] = self.domains[domain](self.env)
                 self.info('done')
             except Exception as err:
-                if type(err) is IOError and err.errno == ENOENT:
+                if isinstance(err, IOError) and err.errno == ENOENT:
                     self.info('not yet created')
                 else:
                     self.info('failed: %s' % err)
@@ -205,11 +254,23 @@ class Sphinx(object):
     def build(self, force_all=False, filenames=None):
         try:
             if force_all:
+                self.builder.compile_all_catalogs()
                 self.builder.build_all()
             elif filenames:
+                self.builder.compile_specific_catalogs(filenames)
                 self.builder.build_specific(filenames)
             else:
+                self.builder.compile_update_catalogs()
                 self.builder.build_update()
+
+            status = (self.statuscode == 0 and
+                      'succeeded' or 'finished with problems')
+            if self._warncount:
+                self.info(bold('build %s, %s warning%s.' %
+                               (status, self._warncount,
+                                self._warncount != 1 and 's' or '')))
+            else:
+                self.info(bold('build %s.' % status))
         except Exception as err:
             # delete the saved env to force a fresh build next time
             envfile = path.join(self.doctreedir, ENV_PICKLE_FILENAME)
@@ -233,6 +294,7 @@ class Sphinx(object):
             wfile.write('\n')
         if hasattr(wfile, 'flush'):
             wfile.flush()
+        self.messagelog.append(message)
 
     def warn(self, message, location=None, prefix='WARNING: '):
         """Emit a warning.
@@ -255,7 +317,7 @@ class Sphinx(object):
             else:
                 location = None
         warntext = location and '%s: %s%s\n' % (location, prefix, message) or \
-                   '%s%s\n' % (prefix, message)
+            '%s%s\n' % (prefix, message)
         if self.warningiserror:
             raise SphinxWarning(warntext)
         self._warncount += 1
@@ -314,6 +376,48 @@ class Sphinx(object):
             message = message % (args or kwargs)
         self._log(lightgray(message), self._status)
 
+    def _display_chunk(chunk):
+        if isinstance(chunk, (list, tuple)):
+            if len(chunk) == 1:
+                return text_type(chunk[0])
+            return '%s .. %s' % (chunk[0], chunk[-1])
+        return text_type(chunk)
+
+    def old_status_iterator(self, iterable, summary, colorfunc=darkgreen,
+                            stringify_func=_display_chunk):
+        l = 0
+        for item in iterable:
+            if l == 0:
+                self.info(bold(summary), nonl=1)
+                l = 1
+            self.info(colorfunc(stringify_func(item)) + ' ', nonl=1)
+            yield item
+        if l == 1:
+            self.info()
+
+    # new version with progress info
+    def status_iterator(self, iterable, summary, colorfunc=darkgreen, length=0,
+                        stringify_func=_display_chunk):
+        if length == 0:
+            for item in self.old_status_iterator(iterable, summary, colorfunc,
+                                                 stringify_func):
+                yield item
+            return
+        l = 0
+        summary = bold(summary)
+        for item in iterable:
+            l += 1
+            s = '%s[%3d%%] %s' % (summary, 100*l/length,
+                                  colorfunc(stringify_func(item)))
+            if self.verbosity:
+                s += '\n'
+            else:
+                s = term_width_line(s)
+            self.info(s, nonl=1)
+            yield item
+        if l > 0:
+            self.info()
+
     # ---- general extensibility interface -------------------------------------
 
     def setup_extension(self, extension):
@@ -324,50 +428,42 @@ class Sphinx(object):
         try:
             mod = __import__(extension, None, None, ['setup'])
         except ImportError as err:
+            self.verbose('Original exception:\n' + traceback.format_exc())
             raise ExtensionError('Could not import extension %s' % extension,
                                  err)
         if not hasattr(mod, 'setup'):
             self.warn('extension %r has no setup() function; is it really '
                       'a Sphinx extension module?' % extension)
+            ext_meta = None
         else:
             try:
-                mod.setup(self)
+                ext_meta = mod.setup(self)
             except VersionRequirementError as err:
                 # add the extension name to the version required
                 raise VersionRequirementError(
                     'The %s extension used by this project needs at least '
                     'Sphinx v%s; it therefore cannot be built with this '
                     'version.' % (extension, err))
+        if ext_meta is None:
+            ext_meta = {}
+        if not ext_meta.get('version'):
+            ext_meta['version'] = 'unknown version'
         self._extensions[extension] = mod
+        self._extension_metadata[extension] = ext_meta
 
     def require_sphinx(self, version):
         # check the Sphinx version if requested
-        if version > sphinx.__version__[:3]:
+        if version > sphinx.__display_version__[:3]:
             raise VersionRequirementError(version)
 
     def import_object(self, objname, source=None):
         """Import an object from a 'module.name' string."""
-        try:
-            module, name = objname.rsplit('.', 1)
-        except ValueError as err:
-            raise ExtensionError('Invalid full object name %s' % objname +
-                                 (source and ' (needed for %s)' % source or ''),
-                                 err)
-        try:
-            return getattr(__import__(module, None, None, [name]), name)
-        except ImportError as err:
-            raise ExtensionError('Could not import %s' % module +
-                                 (source and ' (needed for %s)' % source or ''),
-                                 err)
-        except AttributeError as err:
-            raise ExtensionError('Could not find %s' % objname +
-                                 (source and ' (needed for %s)' % source or ''),
-                                 err)
+        return import_object(objname, source=None)
 
     # event interface
 
     def _validate_event(self, event):
-        event = sys.intern(event)
+        event = intern(event)
         if event not in self._events:
             raise ExtensionError('Unknown event name: %s' % event)
 
@@ -385,18 +481,19 @@ class Sphinx(object):
 
     def disconnect(self, listener_id):
         self.debug('[app] disconnecting event: [id=%s]', listener_id)
-        for event in self._listeners.values():
+        for event in itervalues(self._listeners):
             event.pop(listener_id, None)
 
     def emit(self, event, *args):
         try:
             self.debug2('[app] emitting event: %r%s', event, repr(args)[:100])
-        except Exception:  # not every object likes to be repr()'d (think
-                           # random stuff coming via autodoc)
+        except Exception:
+            # not every object likes to be repr()'d (think
+            # random stuff coming via autodoc)
             pass
         results = []
         if event in self._listeners:
-            for _, callback in self._listeners[event].items():
+            for _, callback in iteritems(self._listeners[event]):
                 results.append(callback(self, *args))
         return results
 
@@ -420,7 +517,7 @@ class Sphinx(object):
             else:
                 raise ExtensionError(
                     'Builder %r already exists (in module %s)' % (
-                    builder.name, self.builderclasses[builder.name].__module__))
+                        builder.name, self.builderclasses[builder.name].__module__))
         self.builderclasses[builder.name] = builder
 
     def add_config_value(self, name, default, rebuild):
@@ -437,16 +534,23 @@ class Sphinx(object):
             raise ExtensionError('Event %r already present' % name)
         self._events[name] = ''
 
+    def set_translator(self, name, translator_class):
+        self.info(bold('A Translator for the %s builder is changed.' % name))
+        self._translators[name] = translator_class
+
     def add_node(self, node, **kwds):
         self.debug('[app] adding node: %r', (node, kwds))
         nodes._add_node_class_names([node.__name__])
-        for key, val in kwds.items():
+        for key, val in iteritems(kwds):
             try:
                 visit, depart = val
             except ValueError:
                 raise ExtensionError('Value for key %r must be a '
                                      '(visit, depart) function tuple' % key)
-            if key == 'html':
+            translator = self._translators.get(key)
+            if translator is not None:
+                pass
+            elif key == 'html':
                 from sphinx.writers.html import HTMLTranslator as translator
             elif key == 'latex':
                 from sphinx.writers.latex import LaTeXTranslator as translator
@@ -586,6 +690,11 @@ class Sphinx(object):
             StandaloneHTMLBuilder.css_files.append(
                 posixpath.join('_static', filename))
 
+    def add_latex_package(self, packagename, options=None):
+        self.debug('[app] adding latex package: %r', packagename)
+        from sphinx.builders.latex import LaTeXBuilder
+        LaTeXBuilder.usepackages.append((packagename, options))
+
     def add_lexer(self, alias, lexer):
         self.debug('[app] adding lexer: %r', (alias, lexer))
         from sphinx.highlighting import lexers
@@ -607,7 +716,7 @@ class Sphinx(object):
     def add_search_language(self, cls):
         self.debug('[app] adding search language: %r', cls)
         from sphinx.search import languages, SearchLanguage
-        assert isinstance(cls, SearchLanguage)
+        assert issubclass(cls, SearchLanguage)
         languages[cls.lang] = cls
 
 
