@@ -5,26 +5,27 @@
 
     The CheckExternalLinksBuilder class.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
-import sys
-import queue
 import socket
+import codecs
 import threading
 from os import path
-from urllib.request import build_opener, Request, HTTPRedirectHandler
-from urllib.parse import unquote
-from urllib.error import HTTPError
-from html.parser import HTMLParser, HTMLParseError
 
+from six.moves import queue
+from six.moves.urllib.request import build_opener, Request, HTTPRedirectHandler
+from six.moves.urllib.parse import unquote, urlsplit, quote
+from six.moves.urllib.error import HTTPError
+from six.moves.html_parser import HTMLParser, HTMLParseError
 from docutils import nodes
 
 from sphinx.builders import Builder
 from sphinx.util.console import purple, red, darkgreen, darkgray, \
     darkred, turquoise
+from sphinx.util.pycompat import TextIOWrapper
 
 
 class RedirectHandler(HTTPRedirectHandler):
@@ -90,7 +91,7 @@ class CheckExternalLinksBuilder(Builder):
     name = 'linkcheck'
 
     def init(self):
-        self.to_ignore = list(map(re.compile, self.app.config.linkcheck_ignore))
+        self.to_ignore = [re.compile(x) for x in self.app.config.linkcheck_ignore]
         self.good = set()
         self.broken = {}
         self.redirected = {}
@@ -111,7 +112,7 @@ class CheckExternalLinksBuilder(Builder):
 
     def check_thread(self):
         kwargs = {}
-        if sys.version_info > (2, 5) and self.app.config.linkcheck_timeout:
+        if self.app.config.linkcheck_timeout:
             kwargs['timeout'] = self.app.config.linkcheck_timeout
 
         def check():
@@ -122,7 +123,7 @@ class CheckExternalLinksBuilder(Builder):
             elif not (uri[0:5] == 'http:' or uri[0:6] == 'https:'):
                 return 'local', '', 0
             elif uri in self.good:
-                return 'working', '', 0
+                return 'working', 'old', 0
             elif uri in self.broken:
                 return 'broken', self.broken[uri], 0
             elif uri in self.redirected:
@@ -131,19 +132,37 @@ class CheckExternalLinksBuilder(Builder):
                 if rex.match(uri):
                     return 'ignored', '', 0
 
+            # split off anchor
             if '#' in uri:
                 req_url, hash = uri.split('#', 1)
             else:
                 req_url = uri
                 hash = None
 
+            # handle non-ASCII URIs
+            try:
+                req_url.encode('ascii')
+            except UnicodeError:
+                split = urlsplit(req_url)
+                req_url = (split[0].encode() + '://' +       # scheme
+                           split[1].encode('idna') +         # netloc
+                           quote(split[2].encode('utf-8')))  # path
+                if split[3]:  # query
+                    req_url += '?' + quote(split[3].encode('utf-8'))
+                # go back to Unicode strings which is required by Python 3
+                # (but now all parts are pure ascii)
+                req_url = req_url.decode('ascii')
+
             # need to actually check the URI
             try:
                 if hash and self.app.config.linkcheck_anchors:
                     # Read the whole document and see if #hash exists
                     req = Request(req_url)
                     f = opener.open(req, **kwargs)
-                    found = check_anchor(f, unquote(hash))
+                    encoding = 'utf-8'
+                    if hasattr(f.headers, 'get_content_charset'):
+                        encoding = f.headers.get_content_charset() or encoding
+                    found = check_anchor(TextIOWrapper(f, encoding), unquote(hash))
                     f.close()
 
                     if not found:
@@ -163,13 +182,20 @@ class CheckExternalLinksBuilder(Builder):
                         req = Request(req_url)
                         f = opener.open(req, **kwargs)
                         f.close()
-
+            except HTTPError as err:
+                if err.code == 401:
+                    # We'll take "Unauthorized" as working.
+                    self.good.add(uri)
+                    return 'working', ' - unauthorized', 0
+                else:
+                    self.broken[uri] = str(err)
+                    return 'broken', str(err), 0
             except Exception as err:
                 self.broken[uri] = str(err)
                 return 'broken', str(err), 0
             if f.url.rstrip('/') == req_url.rstrip('/'):
                 self.good.add(uri)
-                return 'working', 'new', 0
+                return 'working', '', 0
             else:
                 new_url = f.url
                 if hash:
@@ -189,7 +215,7 @@ class CheckExternalLinksBuilder(Builder):
         uri, docname, lineno, status, info, code = result
         if status == 'unchecked':
             return
-        if status == 'working' and info != 'new':
+        if status == 'working' and info == 'old':
             return
         if lineno:
             self.info('(line %4d) ' % lineno, nonl=1)
@@ -199,7 +225,7 @@ class CheckExternalLinksBuilder(Builder):
             self.info(darkgray('-local-   ') + uri)
             self.write_entry('local', docname, lineno, uri)
         elif status == 'working':
-            self.info(darkgreen('ok        ')  + uri)
+            self.info(darkgreen('ok        ')  + uri + info)
         elif status == 'broken':
             self.info(red('broken    ') + uri + red(' - ' + info))
             self.write_entry('broken', docname, lineno, uri + ': ' + info)
@@ -251,7 +277,7 @@ class CheckExternalLinksBuilder(Builder):
             self.app.statuscode = 1
 
     def write_entry(self, what, docname, line, uri):
-        output = open(path.join(self.outdir, 'output.txt'), 'a')
+        output = codecs.open(path.join(self.outdir, 'output.txt'), 'a', 'utf-8')
         output.write("%s:%s: [%s] %s\n" % (self.env.doc2path(docname, None),
                                            line, what, uri))
         output.close()
