@@ -20,12 +20,14 @@
 
     You can also give an explicit caption, e.g. :exmpl:`Foo <foo>`.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from six import iteritems
 from docutils import nodes, utils
 
+import sphinx
 from sphinx.util.nodes import split_explicit_title
 
 
@@ -50,10 +52,13 @@ def make_link_role(base_url, prefix):
         return [pnode], []
     return role
 
+
 def setup_link_roles(app):
-    for name, (base_url, prefix) in app.config.extlinks.items():
+    for name, (base_url, prefix) in iteritems(app.config.extlinks):
         app.add_role(name, make_link_role(base_url, prefix))
 
+
 def setup(app):
     app.add_config_value('extlinks', {}, 'env')
     app.connect('builder-inited', setup_link_roles)
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
