@@ -5,16 +5,17 @@
 
     Locale utilities.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import sys
 import gettext
-import collections
+
+from six import PY3, text_type
+from six.moves import UserString
 
 
-class _TranslationProxy(collections.UserString, object):
+class _TranslationProxy(UserString, object):
     """
     Class for proxy strings from gettext translations.  This is a helper for the
     lazy_* functions from this module.
@@ -32,7 +33,7 @@ class _TranslationProxy(collections.UserString, object):
     def __new__(cls, func, *args):
         if not args:
             # not called with "function" and "arguments", but a plain string
-            return str(func)
+            return text_type(func)
         return object.__new__(cls)
 
     def __getnewargs__(self):
@@ -61,9 +62,10 @@ class _TranslationProxy(collections.UserString, object):
 
     def __bool__(self):
         return bool(self.data)
+    __nonzero__ = __bool__  # for python2 compatibility
 
     def __dir__(self):
-        return dir(str)
+        return dir(text_type)
 
     def __iter__(self):
         return iter(self.data)
@@ -74,8 +76,8 @@ class _TranslationProxy(collections.UserString, object):
     def __str__(self):
         return str(self.data)
 
-    def __str__(self):
-        return str(self.data)
+    def __unicode__(self):
+        return text_type(self.data)
 
     def __add__(self, other):
         return self.data + other
@@ -132,20 +134,22 @@ class _TranslationProxy(collections.UserString, object):
 
     def __repr__(self):
         try:
-            return 'i' + repr(str(self.data))
+            return 'i' + repr(text_type(self.data))
         except:
             return '<%s broken>' % self.__class__.__name__
 
+
 def mygettext(string):
     """Used instead of _ when creating TranslationProxies, because _ is
     not bound yet at that time.
     """
     return _(string)
 
+
 def lazy_gettext(string):
     """A lazy version of `gettext`."""
-    #if isinstance(string, _TranslationProxy):
-    #    return string
+    # if isinstance(string, _TranslationProxy):
+    #     return string
     return _TranslationProxy(mygettext, string)
 
 l_ = lazy_gettext
@@ -183,7 +187,7 @@ pairindextypes = {
 
 translators = {}
 
-if sys.version_info >= (3, 0):
+if PY3:
     def _(message):
         return translators['sphinx'].gettext(message)
 else:
@@ -204,10 +208,18 @@ def init(locale_dirs, language, catalog='sphinx'):
         translator = None
     # the None entry is the system's default locale path
     has_translation = True
+
+    # compile mo files if po file is updated
+    # TODO: remove circular importing
+    from sphinx.util.i18n import find_catalog_source_files
+    for catinfo in find_catalog_source_files(locale_dirs, language, domains=[catalog]):
+        catinfo.write_mo(language)
+
+    # loading
     for dir_ in locale_dirs:
         try:
             trans = gettext.translation(catalog, localedir=dir_,
-                    languages=[language])
+                                        languages=[language])
             if translator is None:
                 translator = trans
             else:
