@@ -5,13 +5,13 @@
 
     Docutils node-related utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
-import sys
 
+from six import text_type
 from docutils import nodes
 
 from sphinx import addnodes
@@ -34,45 +34,91 @@ class WarningStream(object):
 explicit_title_re = re.compile(r'^(.+?)\s*(?<!\x00)<(.*?)>$', re.DOTALL)
 caption_ref_re = explicit_title_re  # b/w compat alias
 
+
+def apply_source_workaround(node):
+    if node.source and node.rawsource:
+        return
+
+    # workaround: nodes.term doesn't have source, line and rawsource
+    # (fixed in Docutils r7495)
+    if isinstance(node, nodes.term):
+        definition_list_item = node.parent
+        if definition_list_item.line is not None:
+            node.source = definition_list_item.source
+            node.line = definition_list_item.line - 1
+            node.rawsource = definition_list_item. \
+                rawsource.split("\n", 2)[0]
+            return
+
+    # workaround: docutils-0.10.0 or older's nodes.caption for nodes.figure
+    # and nodes.title for nodes.admonition doesn't have source, line.
+    # this issue was filed to Docutils tracker:
+    # sf.net/tracker/?func=detail&aid=3599485&group_id=38414&atid=422032
+    # sourceforge.net/p/docutils/patches/108/
+    if (isinstance(node, (
+            nodes.caption,
+            nodes.title,
+            nodes.rubric,
+            nodes.line,
+    ))):
+        node.source = find_source_node(node)
+        node.line = 0  # need fix docutils to get `node.line`
+        return
+
+
 IGNORED_NODES = (
     nodes.Invisible,
     nodes.Inline,
     nodes.literal_block,
     nodes.doctest_block,
-    #XXX there are probably more
+    # XXX there are probably more
 )
-def extract_messages(doctree):
-    """Extract translatable messages from a document tree."""
-    for node in doctree.traverse(nodes.TextElement):
-        # workaround: nodes.term doesn't have source, line and rawsource
-        # (fixed in Docutils r7495)
-        if isinstance(node, nodes.term) and not node.source:
-            definition_list_item = node.parent
-            if definition_list_item.line is not None:
-                node.source = definition_list_item.source
-                node.line = definition_list_item.line - 1
-                node.rawsource = definition_list_item.\
-                                 rawsource.split("\n", 2)[0]
-        # workaround: docutils-0.10.0 or older's nodes.caption for nodes.figure
-        # and nodes.title for nodes.admonition doesn't have source, line.
-        # this issue was filed to Docutils tracker:
-        # sf.net/tracker/?func=detail&aid=3599485&group_id=38414&atid=422032
-        # sourceforge.net/p/docutils/patches/108/
-        if isinstance(node, (nodes.caption, nodes.title, nodes.rubric)) \
-           and not node.source:
-            node.source = find_source_node(node)
-            node.line = 0  #need fix docutils to get `node.line`
+
+
+def is_translatable(node):
+    if isinstance(node, nodes.TextElement):
+        apply_source_workaround(node)
 
         if not node.source:
-            continue # built-in message
+            return False  # built-in message
         if isinstance(node, IGNORED_NODES) and 'translatable' not in node:
-            continue
+            return False
         # <field_name>orphan</field_name>
         # XXX ignore all metadata (== docinfo)
         if isinstance(node, nodes.field_name) and node.children[0] == 'orphan':
-            continue
+            return False
+        return True
+
+    if isinstance(node, nodes.image) and node.get('translatable'):
+        return True
+
+    return False
+
+
+LITERAL_TYPE_NODES = (
+    nodes.literal_block,
+    nodes.doctest_block,
+    nodes.raw,
+)
+IMAGE_TYPE_NODES = (
+    nodes.image,
+)
+
+
+def extract_messages(doctree):
+    """Extract translatable messages from a document tree."""
+    for node in doctree.traverse(is_translatable):
+        if isinstance(node, LITERAL_TYPE_NODES):
+            msg = node.rawsource
+            if not msg:
+                msg = node.astext()
+        elif isinstance(node, IMAGE_TYPE_NODES):
+            msg = '.. image:: %s' % node['uri']
+            if node.get('alt'):
+                msg += '\n   :alt: %s' % node['alt']
+        else:
+            msg = node.rawsource.replace('\n', ' ').strip()
 
-        msg = node.rawsource.replace('\n', ' ').strip()
         # XXX nodes rendering empty are likely a bug in sphinx.addnodes
         if msg:
             yield node, msg
@@ -94,7 +140,7 @@ def traverse_translatable_index(doctree):
     """Traverse translatable index node from a document tree."""
     def is_block_index(node):
         return isinstance(node, addnodes.index) and  \
-            node.get('inline') == False
+            node.get('inline') is False
     for node in doctree.traverse(is_block_index):
         if 'raw_entries' in node:
             entries = node['raw_entries']
@@ -142,6 +188,7 @@ indextypes = [
     'single', 'pair', 'double', 'triple', 'see', 'seealso',
 ]
 
+
 def process_index_entry(entry, targetid):
     indexentries = []
     entry = entry.strip()
@@ -186,12 +233,13 @@ def inline_all_toctrees(builder, docnameset, docname, tree, colorfunc):
     tree = tree.deepcopy()
     for toctreenode in tree.traverse(addnodes.toctree):
         newnodes = []
-        includefiles = list(map(str, toctreenode['includefiles']))
+        includefiles = map(text_type, toctreenode['includefiles'])
         for includefile in includefiles:
             try:
                 builder.info(colorfunc(includefile) + " ", nonl=1)
                 subtree = inline_all_toctrees(builder, docnameset, includefile,
-                    builder.env.get_doctree(includefile), colorfunc)
+                                              builder.env.get_doctree(includefile),
+                                              colorfunc)
                 docnameset.add(includefile)
             except Exception:
                 builder.warn('toctree contains ref to nonexisting '
@@ -200,6 +248,9 @@ def inline_all_toctrees(builder, docnameset, docname, tree, colorfunc):
             else:
                 sof = addnodes.start_of_file(docname=includefile)
                 sof.children = subtree.children
+                for sectionnode in sof.traverse(nodes.section):
+                    if 'docname' not in sectionnode:
+                        sectionnode['docname'] = includefile
                 newnodes.append(sof)
         toctreenode.parent.replace(toctreenode, newnodes)
     return tree
@@ -211,8 +262,8 @@ def make_refnode(builder, fromdocname, todocname, targetid, child, title=None):
     if fromdocname == todocname:
         node['refid'] = targetid
     else:
-        node['refuri'] = (builder.get_relative_uri(fromdocname, todocname)
-                          + '#' + targetid)
+        node['refuri'] = (builder.get_relative_uri(fromdocname, todocname) +
+                          '#' + targetid)
     if title:
         node['reftitle'] = title
     node.append(child)
@@ -223,13 +274,10 @@ def set_source_info(directive, node):
     node.source, node.line = \
         directive.state_machine.get_source_and_line(directive.lineno)
 
+
 def set_role_source_info(inliner, lineno, node):
-    try:
-        node.source, node.line = \
-            inliner.reporter.locator(lineno)
-    except AttributeError:
-        # docutils 0.9+
-        node.source, node.line = inliner.reporter.get_source_and_line(lineno)
+    node.source, node.line = inliner.reporter.get_source_and_line(lineno)
+
 
 # monkey-patch Element.copy to copy the rawsource
 
@@ -237,17 +285,3 @@ def _new_copy(self):
     return self.__class__(self.rawsource, **self.attributes)
 
 nodes.Element.copy = _new_copy
-
-# monkey-patch Element.__repr__ to return str if it returns unicode.
-# Was fixed in docutils since 0.10. See sf.net/p/docutils/bugs/218/.
-
-if sys.version_info < (3,):
-    _element_repr_orig = nodes.Element.__repr__
-
-    def _new_repr(self):
-        s = _element_repr_orig(self)
-        if isinstance(s, str):
-            return s.encode('utf-8')
-        return s
-
-    nodes.Element.__repr__ = _new_repr
