@@ -5,17 +5,16 @@
 
     Base Module for web support functions.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import sys
-import pickle as pickle
 import posixpath
 from os import path
 
+from six.moves import cPickle as pickle
 from jinja2 import Environment, FileSystemLoader
-
 from docutils.core import publish_parts
 
 from sphinx.application import Sphinx
@@ -33,13 +32,13 @@ class WebSupport(object):
     with the web support package should occur through this class.
     """
     def __init__(self,
-                 srcdir=None,     # only required for building
-                 builddir='',     # the dir with data/static/doctrees subdirs
-                 datadir=None,    # defaults to builddir/data
-                 staticdir=None,  # defaults to builddir/static
-                 doctreedir=None, # defaults to builddir/doctrees
-                 search=None,     # defaults to no search
-                 storage=None,    # defaults to SQLite in datadir
+                 srcdir=None,      # only required for building
+                 builddir='',      # the dir with data/static/doctrees subdirs
+                 datadir=None,     # defaults to builddir/data
+                 staticdir=None,   # defaults to builddir/static
+                 doctreedir=None,  # defaults to builddir/doctrees
+                 search=None,      # defaults to no search
+                 storage=None,     # defaults to SQLite in datadir
                  status=sys.stdout,
                  warning=sys.stderr,
                  moderation_callback=None,
@@ -217,7 +216,7 @@ class WebSupport(object):
             'q': q,
             'search_performed': True,
             'search_results': results,
-            'docroot': '../', # XXX
+            'docroot': '../',  # XXX
             '_': _,
         }
         document = {
